/*
 * Copyright (c) 2020 Nordic Semiconductor ASA
 *
 * SPDX-License-Identifier: LicenseRef-Nordic-5-Clause
 */

/** @file
 * @brief Dimmer switch for HA profile implementation.
 */

#include <zephyr/kernel.h>
#include <zephyr/device.h>
#include <zephyr/logging/log.h>
#include <dk_buttons_and_leds.h>
#include <ram_pwrdn.h>

#include <zboss_api.h>
#include <zboss_api_addons.h>
#include <zigbee/zigbee_app_utils.h>
#include <zigbee/zigbee_error_handler.h>
#include <zb_nrf_platform.h>
#include "zb_mem_config_custom.h"
#include "zb_dimmer_switch.h"
#include "zb_smart_controller.h"
#if defined(CONFIG_LIGHT_SWITCH_CONFIGURE_TX_POWER)
#include <osif/mac_platform.h>
#endif

#if CONFIG_ZIGBEE_FOTA
#include <zigbee/zigbee_fota.h>
#include <zephyr/sys/reboot.h>
#include <zephyr/dfu/mcuboot.h>

/* LED indicating OTA Client Activity. */
#define OTA_ACTIVITY_LED          DK_LED2
#endif /* CONFIG_ZIGBEE_FOTA */

#if CONFIG_BT_NUS
#include "nus_cmd.h"

/* LED which indicates that Central is connected. */
#define NUS_STATUS_LED            DK_LED1
/* UART command that will turn on found light bulb(s). */
#define COMMAND_ON                "n"
/**< UART command that will turn off found light bulb(s). */
#define COMMAND_OFF               "f"
/**< UART command that will turn toggle found light bulb(s). */
#define COMMAND_TOGGLE            "t"
/**< UART command that will increase brightness of found light bulb(s). */
#define COMMAND_INCREASE          "i"
/**< UART command that will decrease brightness of found light bulb(s). */
#define COMMAND_DECREASE          "d"
#endif /* CONFIG_BT_NUS */

/* Source endpoint used to control light bulb. */
#define SMART_CONTROLLER_ENDPOINT      1
/* Delay between the light switch startup and light bulb finding procedure. */
#define MATCH_DESC_REQ_START_DELAY K_SECONDS(2)
/* Timeout for finding procedure. */
#define MATCH_DESC_REQ_TIMEOUT     K_SECONDS(5)
/* Find only non-sleepy device. */
#define MATCH_DESC_REQ_ROLE        ZB_NWK_BROADCAST_RX_ON_WHEN_IDLE

/* Do not erase NVRAM to save the network parameters after device reboot or
 * power-off. NOTE: If this option is set to ZB_TRUE then do full device erase
 * for all network devices before running other samples.
 */
#define ERASE_PERSISTENT_CONFIG    ZB_FALSE
/* LED indicating that light switch successfully joind Zigbee network. */
#define ZIGBEE_NETWORK_STATE_LED   DK_LED3
/* LED used for device identification. */
#define IDENTIFY_LED               ZIGBEE_NETWORK_STATE_LED
/* LED indicating that light witch found a light bulb to control. */
#define BULB_FOUND_LED             DK_LED4
/* Button ID used to switch on the light bulb. */
#define BUTTON_ON                  DK_BTN1_MSK
/* Button ID used to switch off the light bulb. */
#define BUTTON_OFF                 DK_BTN2_MSK
/* Dim step size - increases/decreses current level (range 0x000 - 0xfe). */
#define DIMM_STEP                  15
/* Button ID used to enable sleepy behavior. */
#define BUTTON_SLEEPY              DK_BTN3_MSK

/* Button to start Factory Reset */
#define FACTORY_RESET_BUTTON       DK_BTN4_MSK

/* Button used to enter the Identify mode. */
#define IDENTIFY_MODE_BUTTON       DK_BTN4_MSK

/* Transition time for a single step operation in 0.1 sec units.
 * 0xFFFF - immediate change.
 */
#define DIMM_TRANSACTION_TIME      2

/* Time after which the button state is checked again to detect button hold,
 * the dimm command is sent again.
 */
#define BUTTON_LONG_POLL_TMO       K_MSEC(500)





/* Version of the application software (1 byte). */
#define BULB_INIT_BASIC_APP_VERSION     01

/* Version of the implementation of the Zigbee stack (1 byte). */
#define BULB_INIT_BASIC_STACK_VERSION   10

/* Version of the hardware of the device (1 byte). */
#define BULB_INIT_BASIC_HW_VERSION      11

/* Manufacturer name (32 bytes). */
#define BULB_INIT_BASIC_MANUF_NAME      "Linxura"

/* Model number assigned by manufacturer (32-bytes long string). */
#define BULB_INIT_BASIC_MODEL_ID        "Smart Controller"

/* First 8 bytes specify the date of manufacturer of the device
 * in ISO 8601 format (YYYYMMDD). The rest (8 bytes) are manufacturer specific.
 */
#define BULB_INIT_BASIC_DATE_CODE       "20200329"

#define BULB_INIT_BASIC_PH_ENV          ZB_ZCL_BASIC_ENV_UNSPECIFIED


LOG_MODULE_REGISTER(app, LOG_LEVEL_INF);



struct buttons_context {
	uint32_t state;
	atomic_t long_poll;
	struct k_timer alarm;
};

struct zb_device_ctx {
	zb_zcl_basic_attrs_ext_t basic_attr;
	zb_zcl_iaszone_attrs_ext_t iaszone_attr;
};



static struct buttons_context buttons_ctx;
static struct zb_device_ctx dev_ctx;


ZB_ZCL_DECLARE_BASIC_ATTRIB_LIST_EXT(
	basic_attr_list,
	&dev_ctx.basic_attr.zcl_version,
	&dev_ctx.basic_attr.app_version,
	&dev_ctx.basic_attr.stack_version,
	&dev_ctx.basic_attr.hw_version,
	dev_ctx.basic_attr.mf_name,
	dev_ctx.basic_attr.model_id,
	dev_ctx.basic_attr.date_code,
	&dev_ctx.basic_attr.power_source,
	dev_ctx.basic_attr.location_id,
	&dev_ctx.basic_attr.ph_env,
	dev_ctx.basic_attr.sw_ver);

ZB_ZCL_DECLARE_IAS_ZONE_ATTRIB_LIST_EXT(
	ias_zone_attr_list, // 属性列表的唯一标识符
	&dev_ctx.iaszone_attr.zone_state,         // 区域状态变量
	&dev_ctx.iaszone_attr.zone_type,          // 区域类型变量
	&dev_ctx.iaszone_attr.zone_status,        // 区域状态变量
	&dev_ctx.iaszone_attr.number_of_zone_sens_levels_supported, // 支持的灵敏度级别数量
	&dev_ctx.iaszone_attr.current_zone_sens_level, // 当前灵敏度级别
	&dev_ctx.iaszone_attr.ias_cie_address,    // CIE 地址
	&dev_ctx.iaszone_attr.zone_id,            // 区域 ID
	&dev_ctx.iaszone_attr.cie_short_addr,     // CIE 短地址
	&dev_ctx.iaszone_attr.cie_ep              // CIE 端点
);



ZB_DECLARE_SMART_CONTROLLER_CLUSTER_LIST(
	smart_controller_clusters,
	basic_attr_list,
	ias_zone_attr_list);

ZB_DECLARE_SMART_CONTROLLER_EP(
	smart_controller_ep,
	SMART_CONTROLLER_ENDPOINT,
	smart_controller_clusters);

ZBOSS_DECLARE_DEVICE_CTX_1_EP(
	smart_controller_ctx,
	smart_controller_ep);



/* Declare application's device context (list of registered endpoints)
 * for Dimmer Switch device.
 */
#ifndef CONFIG_ZIGBEE_FOTA
//ZBOSS_DECLARE_DEVICE_CTX_1_EP(dimmer_switch_ctx, dimmer_switch_ep);
#else

  #if LIGHT_SWITCH_ENDPOINT == CONFIG_ZIGBEE_FOTA_ENDPOINT
    #error "Light switch and Zigbee OTA endpoints should be different."
  #endif

extern zb_af_endpoint_desc_t zigbee_fota_client_ep;
ZBOSS_DECLARE_DEVICE_CTX_2_EP(dimmer_switch_ctx,
			      zigbee_fota_client_ep,
			      dimmer_switch_ep);
#endif /* CONFIG_ZIGBEE_FOTA */



/**@brief Callback for button events.
 *
 * @param[in]   button_state  Bitmask containing buttons state.
 * @param[in]   has_changed   Bitmask containing buttons that has
 *                            changed their state.
 */
static void button_handler(uint32_t button_state, uint32_t has_changed)
{
	zb_uint16_t cmd_id;
	zb_ret_t zb_err_code;

	zb_uint8_t buf_param ;
    zb_uint16_t new_val = 1 ;
    zb_uint16_t delay = 0;  // 延迟时间（秒）


	/* Inform default signal handler about user input at the device. */
	user_input_indicate();

	check_factory_reset_button(button_state, has_changed);

	switch (has_changed) {
	case BUTTON_ON:
		if (BUTTON_ON & button_state) {
			/* Button changed its state to pressed */
		} else {
			LOG_INF("ON - button changed");
			#if 0
					//cmd_id = ZB_ZCL_CMD_ON_OFF_ON_ID;
				new_val = ZB_ZCL_IAS_ZONE_ZONE_STATUS_ALARM1 ;
				buf_param = zb_buf_get(ZB_TRUE, sizeof(zb_zcl_ias_zone_status_param_t));
				if (buf_param == ZB_BUF_INVALID) {
					LOG_INF("Malloc Buffer Failed \r\n");
					return;
				}
				//zb_zcl_ias_zone_change_status(buf_param);
				zb_zcl_ias_zone_set_status(SMART_CONTROLLER_ENDPOINT,new_val,delay,buf_param);

				zb_buf_free(buf_param);
			#else
			zb_zcl_attr_t *attr_desc;
			attr_desc = zb_zcl_get_attr_desc_a(
				SMART_CONTROLLER_ENDPOINT, 
				ZB_ZCL_CLUSTER_ID_IAS_ZONE, 
				ZB_ZCL_CLUSTER_SERVER_ROLE, 
				ZB_ZCL_ATTR_IAS_ZONE_ZONESTATUS_ID);
			cmd_id = 1;
			zb_uint16_t attr_val = ZB_ZCL_GET_ATTRIBUTE_VAL_16(attr_desc);
			LOG_INF("Old status=%d", attr_val);
			// if(attr_val == cmd_id){
			// 	if(attr_val == 0)
			// 		break;;
			// 	attr_val = 0;
			// 	ZB_ZCL_SET_ATTRIBUTE(
			// 		SMART_CONTROLLER_ENDPOINT,
			// 		ZB_ZCL_CLUSTER_ID_IAS_ZONE,
			// 		ZB_ZCL_CLUSTER_SERVER_ROLE,
			// 		ZB_ZCL_ATTR_IAS_ZONE_ZONESTATUS_ID,
			// 		(zb_uint8_t *)&attr_val,
			// 		ZB_FALSE);
			// 	k_msleep(200);
			// }
			attr_val = cmd_id;
			ZB_ZCL_SET_ATTRIBUTE(
				SMART_CONTROLLER_ENDPOINT,
				ZB_ZCL_CLUSTER_ID_IAS_ZONE,
				ZB_ZCL_CLUSTER_SERVER_ROLE,
				ZB_ZCL_ATTR_IAS_ZONE_ZONESTATUS_ID,
				(zb_uint8_t *)&attr_val,
				ZB_FALSE);
			#endif
		}


		//zb_zcl_ias_zone_change_status(buf_param);
		//zb_zcl_ias_zone_write_attr_hook(SMART_CONTROLLER_ENDPOINT,ZB_ZCL_CLUSTER_ID_IAS_ZONE,&new_val);

		// ZB_ZCL_SET_ATTRIBUTE(
		// 	SMART_CONTROLLER_ENDPOINT,
		// 	ZB_ZCL_CLUSTER_ID_IAS_ZONE,
		// 	ZB_ZCL_CLUSTER_SERVER_ROLE,
		// 	ZB_ZCL_ATTR_IAS_ZONE_ZONESTATUS_ID,
		// 	(zb_uint8_t *)&new_val,
		// 	ZB_FALSE);

		break;
	case BUTTON_OFF:
			if (BUTTON_OFF & button_state) {
				/* Button changed its state to pressed */
			} else {
				LOG_INF("OFF - button changed");
				#if 1
				//cmd_id = ZB_ZCL_CMD_ON_OFF_ON_ID;
			new_val = ZB_ZCL_IAS_ZONE_ZONE_STATUS_ALARM1 ;
			buf_param = zb_buf_get(ZB_TRUE, sizeof(zb_zcl_ias_zone_status_param_t));
			if (buf_param == ZB_BUF_INVALID) {
				LOG_INF("Malloc Buffer Failed \r\n");
				return;
			}
			//zb_zcl_ias_zone_change_status(buf_param);
			zb_zcl_ias_zone_set_status(SMART_CONTROLLER_ENDPOINT,new_val,delay,buf_param);

			zb_buf_free(buf_param);
				#else
						//cmd_id = ZB_ZCL_CMD_ON_OFF_ON_ID;
					new_val = ZB_ZCL_IAS_ZONE_ZONE_STATUS_BATTERY ;
					buf_param = zb_buf_get(ZB_TRUE, sizeof(zb_zcl_ias_zone_notification_param_t));
					if (buf_param == ZB_BUF_INVALID) {
						LOG_INF("Malloc Buffer Failed \r\n");
						return;
					}
				
					//zb_zcl_ias_zone_change_status1(buf_param);
					//zb_zcl_ias_zone_set_status(SMART_CONTROLLER_ENDPOINT,new_val,delay,buf_param);

					zb_buf_free(buf_param);
					#endif
			}
	//	zb_zcl_ias_zone_write_attr_hook(SMART_CONTROLLER_ENDPOINT,ZB_ZCL_CLUSTER_ID_IAS_ZONE,&new_val);
	
		//zb_zcl_ias_zone_set_status(SMART_CONTROLLER_ENDPOINT,new_val,delay,buf_param);
		//cmd_id = ZB_ZCL_CMD_ON_OFF_OFF_ID;

		//zb_zcl_ias_zone_change_status(buf_param);
		// ZB_ZCL_SET_ATTRIBUTE(
		// 	SMART_CONTROLLER_ENDPOINT,
		// 	ZB_ZCL_CLUSTER_ID_IAS_ZONE,
		// 	ZB_ZCL_CLUSTER_SERVER_ROLE,
		// 	ZB_ZCL_ATTR_IAS_ZONE_ZONESTATUS_ID,
		// 	(zb_uint8_t *)&new_val,
		// 	ZB_FALSE);
		break;
	case IDENTIFY_MODE_BUTTON:
		if (IDENTIFY_MODE_BUTTON & button_state) {
			/* Button changed its state to pressed */
		} else {
			/* Button changed its state to released */
			if (was_factory_reset_done()) {
				/* The long press was for Factory Reset */
				LOG_DBG("After Factory Reset - ignore button release");
			} else   {
				/* Button released before Factory Reset */

				/* Start identification mode */
			}
		}
		return;
	default:
		LOG_DBG("Unhandled button");
		return;
	}
	// switch (button_state) {
	// case BUTTON_ON:
	// case BUTTON_OFF:
	// 	LOG_DBG("Button pressed");
	// 	buttons_ctx.state = button_state;

	// 	/* Alarm can be scheduled only once. Next alarm only resets
	// 	 * counting.
	// 	 */
	// 	k_timer_start(&buttons_ctx.alarm, BUTTON_LONG_POLL_TMO,
	// 		      K_NO_WAIT);
	// 	break;
	// case 0:
	// 	LOG_DBG("Button released");

	// 	k_timer_stop(&buttons_ctx.alarm);

	// 	if (atomic_set(&buttons_ctx.long_poll, ZB_FALSE) == ZB_FALSE) {
	// 		/* Allocate output buffer and send on/off command. */
	// 		zb_err_code = zb_buf_get_out_delayed_ext(
	// 			light_switch_send_on_off, cmd_id, 0);
	// 		ZB_ERROR_CHECK(zb_err_code);
	// 	}
	// 	break;
	// default:
	// 	break;
	// }
}

/**@brief Function for initializing LEDs and Buttons. */
static void configure_gpio(void)
{
	int err;

	err = dk_buttons_init(button_handler);
	if (err) {
		LOG_ERR("Cannot init buttons (err: %d)", err);
	}

	err = dk_leds_init();
	if (err) {
		LOG_ERR("Cannot init LEDs (err: %d)", err);
	}
}

static void alarm_timers_init(void)
{
	//k_timer_init(&buttons_ctx.alarm, light_switch_button_handler, NULL);
	//k_timer_init(&bulb_ctx.find_alarm, find_light_bulb_alarm, NULL);
}

/**@brief Function for initializing all clusters attributes. */
static void app_clusters_attr_init(void)
{
		/* Basic cluster attributes data */
		dev_ctx.basic_attr.zcl_version = ZB_ZCL_VERSION;
		dev_ctx.basic_attr.app_version = BULB_INIT_BASIC_APP_VERSION;
		dev_ctx.basic_attr.stack_version = BULB_INIT_BASIC_STACK_VERSION;
		dev_ctx.basic_attr.hw_version = BULB_INIT_BASIC_HW_VERSION;

		/* Use ZB_ZCL_SET_STRING_VAL to set strings, because the first byte
		* should contain string length without trailing zero.
		*
		* For example "test" string will be encoded as:
		*   [(0x4), 't', 'e', 's', 't']
		*/
		ZB_ZCL_SET_STRING_VAL(
			dev_ctx.basic_attr.mf_name,
			BULB_INIT_BASIC_MANUF_NAME,
			ZB_ZCL_STRING_CONST_SIZE(BULB_INIT_BASIC_MANUF_NAME));

		ZB_ZCL_SET_STRING_VAL(
			dev_ctx.basic_attr.model_id,
			BULB_INIT_BASIC_MODEL_ID,
			ZB_ZCL_STRING_CONST_SIZE(BULB_INIT_BASIC_MODEL_ID));

		ZB_ZCL_SET_STRING_VAL(
			dev_ctx.basic_attr.date_code,
			BULB_INIT_BASIC_DATE_CODE,
			ZB_ZCL_STRING_CONST_SIZE(BULB_INIT_BASIC_DATE_CODE));

		dev_ctx.basic_attr.ph_env = BULB_INIT_BASIC_PH_ENV;

		dev_ctx.iaszone_attr.zone_state = ZB_ZCL_IAS_ZONE_ZONESTATE_ENROLLED;
		dev_ctx.iaszone_attr.zone_type = ZB_ZCL_IAS_ZONE_ZONETYPE_MANUF_SPEC;
		dev_ctx.iaszone_attr.zone_status = 0x00;

		// dev_ctx.iaszone_attr.number_of_zone_sens_levels_supported = ZB_ZCL_IAS_ZONE_ZONETYPE_MANUF_SPEC;
		// dev_ctx.iaszone_attr.current_zone_sens_level = ZB_ZCL_IAS_ZONE_ZONETYPE_MANUF_SPEC;

		dev_ctx.iaszone_attr.ias_cie_address = 0x00124B0004A3B4C5;
		dev_ctx.iaszone_attr.zone_id = 1;
		//dev_ctx.iaszone_attr.cie_short_addr = ZB_ZCL_IAS_ZONE_ZONETYPE_MANUF_SPEC;
		dev_ctx.iaszone_attr.cie_ep = 0x01;

		

}

/**@brief Function to toggle the identify LED.
 *
 * @param  bufid  Unused parameter, required by ZBOSS scheduler API.
 */
static void toggle_identify_led(zb_bufid_t bufid)
{
	static int blink_status;

	dk_set_led(IDENTIFY_LED, (++blink_status) % 2);
	ZB_SCHEDULE_APP_ALARM(toggle_identify_led, bufid, ZB_MILLISECONDS_TO_BEACON_INTERVAL(100));
}

/**@brief Function to handle identify notification events on the first endpoint.
 *
 * @param  bufid  Unused parameter, required by ZBOSS scheduler API.
 */
static void identify_cb(zb_bufid_t bufid)
{
	zb_ret_t zb_err_code;

	if (bufid) {
		/* Schedule a self-scheduling function that will toggle the LED. */
		ZB_SCHEDULE_APP_CALLBACK(toggle_identify_led, bufid);
	} else {
		/* Cancel the toggling function alarm and turn off LED. */
		zb_err_code = ZB_SCHEDULE_APP_ALARM_CANCEL(toggle_identify_led, ZB_ALARM_ANY_PARAM);
		ZVUNUSED(zb_err_code);

		/* Update network status/idenitfication LED. */
		if (ZB_JOINED()) {
			dk_set_led_on(ZIGBEE_NETWORK_STATE_LED);
		} else {
			dk_set_led_off(ZIGBEE_NETWORK_STATE_LED);
		}
	}
}



static void process_result_cb(zb_uint8_t param)
{
	LOG_INF("process_result_cb result code %d", param);
	// zb_uint8_t	new_val = 1;
	// 	ZB_ZCL_SET_ATTRIBUTE(
	// 		SMART_CONTROLLER_ENDPOINT,
	// 		ZB_ZCL_CLUSTER_ID_IAS_ZONE,
	// 		ZB_ZCL_CLUSTER_SERVER_ROLE,
	// 		ZB_ZCL_ATTR_IAS_ZONE_ZONESTATUS_ID,
	// 		(zb_uint8_t *)&new_val,
	// 		ZB_FALSE);

}

static  void general_cb(zb_uint8_t param, zb_uint16_t general_val)
{
	
	LOG_INF("general_cb %d %d", param,general_val);
	return NULL;
}





/**@brief Zigbee stack event handler.
 *
 * @param[in]   bufid   Reference to the Zigbee stack buffer
 *                      used to pass signal.
 */
void zboss_signal_handler(zb_bufid_t bufid)
{
	zb_zdo_app_signal_hdr_t *sig_hndler = NULL;
	zb_zdo_app_signal_type_t sig = zb_get_app_signal(bufid, &sig_hndler);
	zb_ret_t status = ZB_GET_APP_SIGNAL_STATUS(bufid);

	/* Update network status LED. */
	zigbee_led_status_update(bufid, ZIGBEE_NETWORK_STATE_LED);

#ifdef CONFIG_ZIGBEE_FOTA
	/* Pass signal to the OTA client implementation. */
	zigbee_fota_signal_handler(bufid);
#endif /* CONFIG_ZIGBEE_FOTA */

	switch (sig) {
	case ZB_BDB_SIGNAL_DEVICE_REBOOT:
	/* fall-through */
	case ZB_BDB_SIGNAL_STEERING:
		/* Call default signal handler. */
		ZB_ERROR_CHECK(zigbee_default_signal_handler(bufid));
		// if (status == RET_OK) {
		// 	/* Check the light device address. */
		// 	if (bulb_ctx.short_addr == 0xFFFF) {
		// 		k_timer_start(&bulb_ctx.find_alarm,
		// 			      MATCH_DESC_REQ_START_DELAY,
		// 			      MATCH_DESC_REQ_TIMEOUT);
		// 	}
		// }
		break;
	case ZB_ZDO_SIGNAL_LEAVE:
		/* If device leaves the network, reset bulb short_addr. */
		// if (status == RET_OK) {
		// 	zb_zdo_signal_leave_params_t *leave_params =
		// 		ZB_ZDO_SIGNAL_GET_PARAMS(sig_hndler, zb_zdo_signal_leave_params_t);

		// 	if (leave_params->leave_type == ZB_NWK_LEAVE_TYPE_RESET) {
		// 		bulb_ctx.short_addr = 0xFFFF;
		// 	}
		// }
		/* Call default signal handler. */
		ZB_ERROR_CHECK(zigbee_default_signal_handler(bufid));
		break;

	default:
		/* Call default signal handler. */
		ZB_ERROR_CHECK(zigbee_default_signal_handler(bufid));
		break;
	}

	if (bufid) {
		zb_buf_free(bufid);
	}
}

#if CONFIG_BT_NUS

static void turn_on_cmd(struct k_work *item)
{
	ARG_UNUSED(item);
	zb_buf_get_out_delayed_ext(light_switch_send_on_off,
				   ZB_ZCL_CMD_ON_OFF_ON_ID, 0);
}

static void turn_off_cmd(struct k_work *item)
{
	ARG_UNUSED(item);
	zb_buf_get_out_delayed_ext(light_switch_send_on_off,
				   ZB_ZCL_CMD_ON_OFF_OFF_ID, 0);
}

static void toggle_cmd(struct k_work *item)
{
	ARG_UNUSED(item);
	zb_buf_get_out_delayed_ext(light_switch_send_on_off,
				   ZB_ZCL_CMD_ON_OFF_TOGGLE_ID, 0);
}

static void increase_cmd(struct k_work *item)
{
	ARG_UNUSED(item);
	zb_buf_get_out_delayed_ext(light_switch_send_step,
				   ZB_ZCL_LEVEL_CONTROL_STEP_MODE_UP, 0);
}

static void decrease_cmd(struct k_work *item)
{
	ARG_UNUSED(item);
	zb_buf_get_out_delayed_ext(light_switch_send_step,
				   ZB_ZCL_LEVEL_CONTROL_STEP_MODE_DOWN, 0);
}

static void on_nus_connect(struct k_work *item)
{
	ARG_UNUSED(item);
	dk_set_led_on(NUS_STATUS_LED);
}

static void on_nus_disconnect(struct k_work *item)
{
	ARG_UNUSED(item);
	dk_set_led_off(NUS_STATUS_LED);
}

static struct nus_entry commands[] = {
	NUS_COMMAND(COMMAND_ON, turn_on_cmd),
	NUS_COMMAND(COMMAND_OFF, turn_off_cmd),
	NUS_COMMAND(COMMAND_TOGGLE, toggle_cmd),
	NUS_COMMAND(COMMAND_INCREASE, increase_cmd),
	NUS_COMMAND(COMMAND_DECREASE, decrease_cmd),
	NUS_COMMAND(NULL, NULL),
};

#endif /* CONFIG_BT_NUS */

#if defined(CONFIG_LIGHT_SWITCH_CONFIGURE_TX_POWER)

/**@brief Callback for TX power setting result.
 *
 * @param  param  Reference to Zigbee stack buffer.
 */
static void tx_power_cb(zb_bufid_t param)
{
	char *status = NULL;
	zb_tx_power_params_t *power_params = zb_buf_begin(param);

	switch (power_params->status) {
	case RET_OK:
		status = "success";
		break;
	case RET_INVALID_PARAMETER_1:
		status = "invalid page";
		break;
	case RET_INVALID_PARAMETER_2:
		status = "invalid channel";
		break;
	case RET_INVALID_PARAMETER_3:
		status = "invalid tx power";
		break;
	default:
		status = "unknown";
		break;
	}

	LOG_INF("Zigbee transceiver tx power set result (pg: %d, ch: %d, pw: %d): %s",
		power_params->page, power_params->channel, power_params->tx_power, status);

	zb_buf_free(param);
}

/**@brief Function for scheduling TX power set.
 *
 * @param  param    Reference to Zigbee stack buffer.
 * @param  channel  Channel number.
 */
static void schedule_tx_power_set(zb_bufid_t param, zb_uint16_t channel)
{
	zb_tx_power_params_t *power_params;

	power_params = zb_buf_initial_alloc(param, sizeof(zb_tx_power_params_t));

	power_params->page = ZB_CHANNEL_PAGE0_2_4_GHZ;
	power_params->channel = channel;
	power_params->tx_power = CONFIG_LIGHT_SWITCH_TX_POWER;
	power_params->cb = tx_power_cb;

	ZB_SCHEDULE_APP_CALLBACK(zb_set_tx_power_async, param);
}

/**@brief Function for setting TX power for configured channels.
 */
void set_tx_power(void)
{
	zb_ret_t ret;
	uint32_t channel_mask;
	uint8_t channel = ZB_TRANSCEIVER_START_CHANNEL_NUMBER;

#if defined(CONFIG_ZIGBEE_CHANNEL_SELECTION_MODE_SINGLE)
	channel_mask = 1 << CONFIG_ZIGBEE_CHANNEL;
#elif defined(CONFIG_ZIGBEE_CHANNEL_SELECTION_MODE_MULTI)
	channel_mask = CONFIG_ZIGBEE_CHANNEL_MASK;
#endif

	for (; channel <= ZB_TRANSCEIVER_MAX_CHANNEL_NUMBER; channel++) {
		if (channel_mask & (1 << channel)) {
			LOG_INF("Setting tx power for channel %d: %d dBm", channel,
				CONFIG_LIGHT_SWITCH_TX_POWER);
			ret = zb_buf_get_out_delayed_ext(schedule_tx_power_set, channel, 0);
			if (ret != RET_OK) {
				LOG_ERR("Failed to set tx power for channel %d, error %d",
					channel, ret);
			}
		}
	}
}

#endif /* CONFIG_LIGHT_SWITCH_CONFIGURE_TX_POWER */

static void zcl_device_cb(zb_bufid_t bufid)
{
	zb_uint8_t cluster_id;
	zb_uint8_t attr_id;
	zb_zcl_device_callback_param_t  *device_cb_param =
		ZB_BUF_GET_PARAM(bufid, zb_zcl_device_callback_param_t);

	LOG_INF("%s id %hd", __func__, device_cb_param->device_cb_id);

	/* Set default response value. */
	device_cb_param->status = RET_OK;


	LOG_INF("%s status: %hd", __func__, device_cb_param->status);
}

void test1(void)
{
	// zb_zcl_attr_t *attr_desc;
	// attr_desc = zb_zcl_get_attr_desc_a(
	// 	SMART_CONTROLLER_ENDPOINT, 
	// 	ZB_ZCL_CLUSTER_ID_IAS_ZONE, 
	// 	ZB_ZCL_CLUSTER_SERVER_ROLE, 
	// 	ZB_ZCL_ATTR_IAS_ZONE_ZONESTATUS_ID);

	// zb_zcl_reporting_info_t *rep_info;
	// rep_info = zb_zcl_find_reporting_info_manuf(
	// 	SMART_CONTROLLER_ENDPOINT,
	// 	ZB_ZCL_CLUSTER_ID_IAS_ZONE,
	// 	ZB_ZCL_CLUSTER_SERVER_ROLE,
	// 	ZB_ZCL_ATTR_IAS_ZONE_ZONESTATUS_ID,
	// 	attr_desc->manuf_code);

	// rep_info->u.send_info.max_interval = 1;
}

int main(void)
{
	LOG_INF("Starting ZBOSS Light Switch example 1");

	/* Initialize. */
	configure_gpio();
	alarm_timers_init();
	register_factory_reset_button(FACTORY_RESET_BUTTON);

	zigbee_erase_persistent_storage(ERASE_PERSISTENT_CONFIG);
	zb_set_ed_timeout(ED_AGING_TIMEOUT_64MIN);
	zb_set_keepalive_timeout(ZB_MILLISECONDS_TO_BEACON_INTERVAL(3000));

	/* Set default bulb short_addr. */
	//bulb_ctx.short_addr = 0xFFFF;

	/* If "sleepy button" is defined, check its state during Zigbee
	 * initialization and enable sleepy behavior at device if defined button
	 * is pressed.
	 */
#if defined BUTTON_SLEEPY
	if (dk_get_buttons() & BUTTON_SLEEPY) {
		zigbee_configure_sleepy_behavior(true);
	}
#endif

	/* Power off unused sections of RAM to lower device power consumption. */
	if (IS_ENABLED(CONFIG_RAM_POWER_DOWN_LIBRARY)) {
		power_down_unused_ram();
	}

	ZB_ZCL_REGISTER_DEVICE_CB(zcl_device_cb);
	/* Register dimmer switch device context (endpoints). */
	ZB_AF_REGISTER_DEVICE_CTX(&smart_controller_ctx);
	app_clusters_attr_init();
	zb_zcl_ias_zone_register_cb(SMART_CONTROLLER_ENDPOINT, process_result_cb,general_cb);


#ifdef CONFIG_ZIGBEE_FOTA
	ZB_AF_SET_IDENTIFY_NOTIFICATION_HANDLER(CONFIG_ZIGBEE_FOTA_ENDPOINT, identify_cb);
#endif /* CONFIG_ZIGBEE_FOTA */

#if defined(CONFIG_LIGHT_SWITCH_CONFIGURE_TX_POWER)
	set_tx_power();
#endif /* CONFIG_LIGHT_SWITCH_CONFIGURE_TX_POWER */

test1();

	/* Start Zigbee default thread. */
	zigbee_enable();

#if CONFIG_BT_NUS
	/* Initialize NUS command service. */
	nus_cmd_init(on_nus_connect, on_nus_disconnect, commands);
#endif /* CONFIG_BT_NUS */

	LOG_INF("ZBOSS Light Switch example started");

	while (1) {
		// k_sleep(K_FOREVER);
		k_sleep(K_MSEC(1000));
		zb_zcl_attr_t *attr_desc;
		attr_desc = zb_zcl_get_attr_desc_a(
			SMART_CONTROLLER_ENDPOINT, 
			ZB_ZCL_CLUSTER_ID_IAS_ZONE, 
			ZB_ZCL_CLUSTER_SERVER_ROLE, 
			ZB_ZCL_ATTR_IAS_ZONE_ZONESTATUS_ID);
		zb_zcl_reporting_info_t *rep_info;
		rep_info = zb_zcl_find_reporting_info_manuf(
			SMART_CONTROLLER_ENDPOINT,
			ZB_ZCL_CLUSTER_ID_IAS_ZONE,
			ZB_ZCL_CLUSTER_SERVER_ROLE,
			ZB_ZCL_ATTR_IAS_ZONE_ZONESTATUS_ID,
			attr_desc->manuf_code);
		LOG_INF("reporting flag=0x%X, max_interval=0x%X", rep_info->flags, rep_info->u.send_info.max_interval);
	}
}
